@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -65,17 +65,17 @@ void Basic_buffer::add_line( const char * const buf, const int len )
   }
 
 
-     // If all lines end with CR-LF, assume DOS file
+// If all the complete lines end with CR-LF, assume DOS file
 bool Basic_buffer::detect_and_remove_cr()
   {
-  if( lines() <= 1 ) return false;
+  if( lines() <= 1 ) return false;			// empty buffer
   for( int i = 0; i < last_line(); ++i )
     {
     const std::string & s = data[i];
     const int size = s.size();
     if( size < 2 || s[size-2] != '\r' ) return false;
     }
-  for( int i = 0; i < last_line(); ++i )
+  for( int i = 0; i < last_line(); ++i )	// remove the CR before each LF
     data[i].erase( data[i].size() - 2, 1 );
   return true;
   }
@@ -362,19 +362,17 @@ Point Basic_buffer::eol( const Point & p ) const
   }
 
 
-// returns position of first non whitespace character in line (eol if none)
+// returns position of first non-blank character in line (eol if none)
 Point Basic_buffer::bot( const Point & p ) const
   {
   Point p1 = bol( p );
   const std::string & s = data[p1.line];
-  while( p1.col < (int)s.size() &&
-         ISO_8859::isspace( s[p1.col] ) && s[p1.col] != '\n' )
-    ++p1.col;
+  while( p1.col < (int)s.size() && ISO_8859::isblank( s[p1.col] ) ) ++p1.col;
   return p1;
   }
 
 
-// returns position following last non whitespace character in line (bol if none)
+// returns position following last non-whitespace character in line (bol if none)
 Point Basic_buffer::eot( const Point & p ) const
   {
   Point p1 = eol( p );
@@ -384,7 +382,7 @@ Point Basic_buffer::eot( const Point & p ) const
   }
 
 
-// begin of paragraph
+// begin of paragraph (bol of first non-blank line in paragraph)
 Point Basic_buffer::bop( const Point & p ) const
   {
   Point p1 = p;
@@ -393,7 +391,7 @@ Point Basic_buffer::bop( const Point & p ) const
   }
 
 
-// end of paragraph
+// end of paragraph (bol of blank line after paragraph)
 Point Basic_buffer::eop( const Point & p ) const
   {
   Point p1 = p;
@@ -402,11 +400,11 @@ Point Basic_buffer::eop( const Point & p ) const
   }
 
 
-// Move 'p' to the matching delimiter.
-// If (force && forward), fail if [p] is closing delimiter.
-// If (force && !forward), fail if [p] is opening delimiter.
-// Returns +1 if done, 0 if [p] is not a delimiter, -1 if no matching delimiter.
-//
+/* Move 'p' to the matching delimiter.
+   If (force && forward), fail if [p] is closing delimiter.
+   If (force && !forward), fail if [p] is opening delimiter.
+   Returns +1 if done, 0 if [p] is not a delimiter, -1 if no matching delimiter.
+*/
 int Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
                                              const bool force ) const
   {
@@ -516,9 +514,9 @@ int Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
   }
 
 
-// Search forward from 'p' for 'pattern' (Boyer-Moore algorithm)
-// If found, set 'p' after found substring
-//
+/* Search forward from 'p' for 'pattern' (Boyer-Moore algorithm)
+   If found, set 'p' after the substring found
+*/
 bool Basic_buffer::find_text( Point & p, const std::string & text, const bool icase ) const
   {
   if( text.empty() ) return true;
@@ -556,9 +554,9 @@ bool Basic_buffer::find_text( Point & p, const std::string & text, const bool ic
   }
 
 
-// Search backward from 'p-1' for 'pattern' (Boyer-Moore algorithm)
-// If found, set 'p' to the first char of found substring
-//
+/* Search backward from 'p-1' for 'pattern' (Boyer-Moore algorithm)
+   If found, set 'p' to the first char of the substring found
+*/
 bool Basic_buffer::rfind_text( Point & p, const std::string & text, const bool icase ) const
   {
   if( text.empty() ) return true;
