@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -56,7 +56,7 @@ std::vector< Point * > & get_marks_and_pointers( Buffer & b )
   }
 
 
-     // adjust marks and pointers after a block insert/delete/replace
+// adjust marks and pointers after a block insert/delete/replace
 void adjust_marks_and_pointers( Buffer & b, const Point & p1,
                                 const Point & p2_old, const Point & p2_new,
                                 const Point * const p_ignore = 0 )
@@ -181,10 +181,11 @@ Buffer::Buffer( const std::string * const namep, const bool read_only )
   : backuped( false ), modified_( false ), saved( false ), crlf( false )
   {
   FILE * f = 0;
+  const bool valid_namep = ( namep && namep->size() );
   errno = 0;
-  if( ( !namep || namep->empty() || *namep == "-" ) && !isatty( fileno( stdin ) ) )
+  if( ( !valid_namep || *namep == "-" ) && !isatty( fileno( stdin ) ) )
     f = stdin;
-  else if( namep && namep->size() )
+  else if( valid_namep )
     {
     name_ = *namep;
     if( read_only ||
@@ -198,6 +199,8 @@ Buffer::Buffer( const std::string * const namep, const bool read_only )
 
   const int bufsize = 65536;
   char buf[bufsize];
+  bool aborted = false;
+  const bool interactive = Screen::height() > 0;
   bool file_eof = true;
   while( file_eof )
     {
@@ -212,6 +215,8 @@ Buffer::Buffer( const std::string * const namep, const bool read_only )
       catch( ... ) { file_eof = false; break; }
       pos = newpos + 1;
       }
+    if( interactive && Screen::kbhit() >= 0 && Screen::wait_kbhit() == 3 )
+      { file_eof = false; aborted = true; }
     }
 
   const bool file_error = std::ferror( f );
@@ -219,6 +224,7 @@ Buffer::Buffer( const std::string * const namep, const bool read_only )
   if( f != stdin ) std::fclose( f );
   else if( !ttyname( 1 ) || !std::freopen( ttyname( 1 ), "r", stdin ) )
     throw Error( "Error opening terminal for input" );
+  if( aborted ) throw Error( "Aborted" );
   if( file_error ) throw Error( "Error reading file" );
   if( !file_eof ) throw Error( "Not enough memory" );
   crlf = detect_and_remove_cr();
@@ -253,46 +259,53 @@ int Buffer::set_marks()
   }
 
 
-bool Buffer::save( const std::string * namep, const bool set_name,
+/* *namep  name   save_to   action
+     ""     ""       x      error              (no names)
+     ""      x     true     error              (no name to save to)
+     x     valid   false    save               (if !read_only)
+   valid    ""     false    save_and_set_name  (if !read_only)
+   valid     x     true     save_to            (save to 'other' file)
+*/
+bool Buffer::save( const std::string * namep, bool save_to,
                    const bool append, Point p1, Point p2 ) const
   {
-  const bool save_as = ( namep && namep->size() );
-  if( options.read_only && ( !save_as || name_ == *namep || name_.empty() ) )
-    return false;
+  const bool valid_namep = ( namep && namep->size() );
+  const bool had_name = ( name_.size() > 0 );
+  if( !valid_namep && !had_name ) return false;		// no names
+  if( !valid_namep && save_to ) return false;		// no name to save to
+
+  if( !save_to && had_name && valid_namep && name_ != *namep ) save_to = true;
+  if( !save_to && options.read_only ) return false;
+
   int mode = 0;
   struct stat st;
   if( append ) mode = S_IRUSR | S_IWUSR;
-  else if( ( save_as && stat( namep->c_str(), &st ) == 0 ) ||
-           ( name_.size() && ( !save_as || name_ != *namep ) &&
+  else if( ( valid_namep && stat( namep->c_str(), &st ) == 0 ) ||
+           ( had_name && ( !valid_namep || name_ != *namep ) &&
              stat( name_.c_str(), &st ) == 0 ) ) mode = st.st_mode;
 
-  const bool had_name = ( name_.size() > 0 );
-  bool same_name = set_name;
-  if( set_name )
+  // ask if not saving to buffer's own file except for emergency save
+  if( ( save_to || !had_name ) && !append )
     {
-    if( save_as )
+    FILE * f = std::fopen( namep->c_str(), "r+" );
+    if( f )
       {
-      if( name_ != *namep )
-        {
-        FILE * f = std::fopen( namep->c_str(), "r+" );
-        if( f )
-          {
-          std::fclose( f );
-          int key = Screen::show_message( "File exists. Overwrite ? (y/N) ",
-                                          true, true );
-          if( key != 'y' && key != 'Y' ) return false;
-          }
-        }
-      if( name_.empty() ) name_ = *namep;
-      else if( name_ != *namep ) same_name = false;
+      std::fclose( f );
+      int key = Screen::show_message( "File exists. Overwrite ? (y/N) ",
+                                      true, true );
+      if( key != 'y' && key != 'Y' ) return false;
       }
-    else { if( name_.empty() ) return false; else namep = &name_; }
     }
-  if( !namep || ( same_name && options.read_only ) ) return false;
+  if( !save_to )			// fill missing name_ or namep
+    {
+    if( valid_namep ) { if( !had_name ) name_ = *namep; }
+    else if( had_name ) namep = &name_;
+    }
+  if( !namep ) return false;		// always save to file name '*namep'
 
   if( !Menu::make_path( *namep ) )
     { if( !had_name ) { name_.clear(); } throw Error( "Error creating path" ); }
-  if( same_name && !saved && !append )
+  if( !save_to && !saved && !append )
     {
     if( RC::editor_options().backup && !backuped )
       {
@@ -313,7 +326,7 @@ bool Buffer::save( const std::string * namep, const bool set_name,
   if( !this->pisvalid( p2 ) ) p2 = this->eof();
   if( !crlf )
     for( Point p = p1; p < p2; ) { std::fputc( pgetc( p ), f ); }
-  else for( Point p = p1; p < p2; )
+  else for( Point p = p1; p < p2; )	// put a CR before each LF
     {
     const int ch = pgetc( p );
     if( ch == '\n' ) std::fputc( '\r', f );
@@ -321,7 +334,7 @@ bool Buffer::save( const std::string * namep, const bool set_name,
     }
   if( std::fclose( f ) != 0 )
     { if( !had_name ) { name_.clear(); } throw Error( "Error closing file" ); }
-  if( same_name )
+  if( !save_to )
     {
     for( int i = change_history.records() - 1; i >= 0; --i )
       if( change_history.record_vector[i].modified == false )
@@ -415,13 +428,13 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
         if( pv[i]->line == p0.line && pv[i]->col > p0.col && pv[i] != &p )
           ++pv[i]->col;
       if( options.word_wrap && to_cursor( p ).col > options.rmargin() + 1 &&
-          !ISO_8859::isblank_( ch ) )
+          !ISO_8859::isblank( ch ) )
         {
         const Point pt = bot( p0 );
         Point p2 = p0;
-        while( p2 > pt && pprev( p2 ) && !ISO_8859::isblank_( (*this)[p2] ) );
+        while( p2 > pt && pprev( p2 ) && !ISO_8859::isblank( (*this)[p2] ) );
         Point p1 = p2;
-        while( p1 > pt && pprev( p1 ) && ISO_8859::isblank_( (*this)[p1] ) );
+        while( p1 > pt && pprev( p1 ) && ISO_8859::isblank( (*this)[p1] ) );
         if( p1 >= pt && p1 < p2 && pnext( p1 ) && pnext( p2 ) )
           {
           reset_appendable(); pdelb( p1, p2 ); p2 = p1;
