@@ -1,26 +1,26 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-// There are two uses for 'Points'; cursors and pointers.
-// 'cursors' are unbounded non-negative coordinates.
-// 'pointers' always point to a valid character in 'Buffer' or to eof.
-// They are different because characters can be more than a column wide,
-// and cursors can go beyond the last character in a buffer line.
-//
+/* There are two uses for 'Points'; cursors and pointers.
+   'cursors' are unbounded non-negative coordinates.
+   'pointers' always point to a valid character in 'Buffer' or to eof.
+   They are different because characters can be more than a column wide,
+   and cursors can go beyond the last character in a buffer line.
+*/
 struct Point
   {
   int line, col;
@@ -54,9 +54,9 @@ struct Point
   };
 
 
-// All the lines are newline terminated except the last one, whose
-// next-to-last character position ( data[last_line()].end() ) is eof().
-//
+/* All the lines are newline terminated except the last one, whose
+   next-to-last character position ( data[last_line()].end() ) is eof().
+*/
 class Basic_buffer
   {
   std::vector< std::string > data;		// the lines
@@ -72,7 +72,8 @@ public:
   Basic_buffer() : data( 1 ) {}
   Basic_buffer( const Basic_buffer & b, const Point & p1, const Point & p2 );
 
-  bool blank( const int line ) const;	// true if line is blank
+  // true if line is blank (invalid, empty, or containing only whitespace)
+  bool blank( const int line ) const;
   int  characters( const int line ) const;
   bool compare_lines( const int line1, const int line2 ) const
     { return ( data[line1] == data[line2] ); }
@@ -91,14 +92,14 @@ public:
   bool pseek( Point & p, const int n ) const;
   bool pvalid( Point & p ) const;	// make p valid, return old p status
 
-  // insert block b.[p1,p2) at p and move next
+       // insert block b.[p1,p2) at p and move next
   bool pputb( Point & p, const Basic_buffer & b,
               const Point & p1, const Point & p2 );
-  // insert char and move next
+       // insert char and move next
   bool pputc( Point & p, const unsigned char ch );
-  // change char and move next
+       // change char and move next
   bool pchgc( Point & p, const unsigned char ch );
-  // delete block at [p1,p2)
+       // delete block at [p1,p2)
   bool pdelb( const Point & p1, const Point & p2 );
   bool capitalize( const Point & p1, const Point & p2 );
   bool to_lowercase( const Point & p1, const Point & p2 );
@@ -198,11 +199,12 @@ public:
     {
     int lmargin_, rmargin_;
   public:
-    bool auto_indent, overwrite, read_only, word_wrap;
+    bool auto_indent, overwrite, read_only, two_spaces, word_wrap;
 
     Options()
       : lmargin_( 0 ), rmargin_( 75 ), auto_indent( false ),
-      overwrite( false ), read_only( false ), word_wrap( false ) {}
+      overwrite( false ), read_only( false ), two_spaces( false ),
+      word_wrap( false ) {}
 
     void reset() { *this = Options(); }
     int lmargin() const { return lmargin_; }
@@ -211,7 +213,7 @@ public:
     bool set_rmargin( const int r );
     };
 
-  int handnum;			// used by delete_all_handles
+  int handnum;			// used by delete_all_handles, show_status
   Options options;
   Point mark[10];		// user bookmarks
   bool crlf;
@@ -219,7 +221,7 @@ public:
   explicit Buffer( const std::string * const namep = 0,
                    const bool read_only = false );
   int set_marks();		// set initial or extended user bookmarks
-  bool save( const std::string * namep = 0, const bool set_name = true,
+  bool save( const std::string * namep = 0, bool save_to = false,
              const bool append = false, Point p1 = Point(), Point p2 = Point() ) const;
 
   long character_offset( const Point & p, int & percent ) const;
@@ -228,6 +230,8 @@ public:
   const std::string & name() const { return name_; }
   const std::string & nename() const			// non-empty name
     { if( name_.size() ) return name_; else return unnamed; }
+  void mark_as_unmodified() const { modified_ = false; } // for emergency_save
+  bool precious() const { return modified_ && !empty(); }
 
        // insert block b.[p1,p2) at p and move next
   bool pputb( Point & p, const Basic_buffer & b, const Point & p1,
