@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cstdio>
@@ -36,14 +36,14 @@ int last_visited_;			// absolute index for last_visited
 std::vector< Buffer_handle > data;
 
 
-// Saves modified buffers if editor dies
+// Saves modified non-empty buffers if editor dies
 //
 void emergency_save()
   {
   if( !handles() ) return;
   int i = 0;
   for( ; i < handles(); ++i )
-    if( data[i].buffer().modified() ) break;
+    if( data[i].buffer().precious() ) break;
   if( i >= handles() ) return;
 
   const std::string dead_name( "DEADMOE" );
@@ -55,11 +55,13 @@ void emergency_save()
                 std::ctime( &t ) );
   std::fflush( f );
   for( i = 0; i < handles(); ++i )
-    if( data[i].buffer().modified() )
+    if( data[i].buffer().precious() )
       {
       std::fprintf( f, "\n*** File '%s'\n", data[i].buffer().nename().c_str() );
       std::fflush( f );
-      data[i].buffer().save( &dead_name, false, true );
+      data[i].buffer().save( &dead_name, true, true );
+      // save buffer once even if it has multiple handles
+      data[i].buffer().mark_as_unmodified();
       }
   std::fclose( f );
   }
@@ -88,28 +90,32 @@ const char * Bufhandle_vector::save_all_named()
   Screen::show_status_lines();
   int bufsize = 0;
   char * const buf = Screen::line_buf( &bufsize );
-  snprintf( buf, bufsize, "%s%d named buffer(s) saved",
-            done ? "" : "Only ", count );
+  snprintf( buf, bufsize, "%s%d named buffer%s saved",
+            done ? "" : "Only ", count, ( count == 1 ) ? "" : "s" );
   return buf;
   }
 
 
 const char * Bufhandle_vector::show_status()
   {
-  int mcount = 0, ucount = 0;
+  int bcount = 0, mcount = 0, ucount = 0;
 
+  for( int i = 0; i < handles(); ++i ) data[i].buffer().handnum = i;
   for( int i = 0; i < handles(); ++i )
-    {
-    if( data[i].buffer().modified() ) ++mcount;
-    if( data[i].buffer().name().empty() ) ++ucount;
-    }
+    if( data[i].buffer().handnum == i )	// last handle for this buffer
+      {
+      ++bcount;				// buffer counter
+      if( data[i].buffer().modified() ) ++mcount;
+      if( data[i].buffer().name().empty() ) ++ucount;
+      }
 
   Screen::show_status_lines();
+  if( bcount == 1 ) return "** This is the only buffer being edited **";
   int bufsize = 0;
   char * const buf = Screen::line_buf( &bufsize );
   snprintf( buf, bufsize,
-            "**  %d modified and  %d unnamed out of  %d total buffer(s) **",
-            mcount, ucount, handles() );
+            "**  %d modified and  %d unnamed out of  %d total buffers **",
+            mcount, ucount, bcount );
   return buf;
   }
 
@@ -182,7 +188,7 @@ int Bufhandle_vector::find_or_add_handle( const std::string * const namep,
   }
 
 
-     // Add empty buffer if no files given or stdin is not a tty
+// Add empty buffer if no files given or stdin is not a tty
 void Bufhandle_vector::add_handle_if_pending_input_or_empty( const int line,
                                                              const int col )
   {
@@ -225,7 +231,7 @@ void Bufhandle_vector::delete_all_handles()
   {
   for( int i = 0; i < handles(); ++i ) data[i].buffer().handnum = i;
   for( int i = handles() - 1; i >= 0; --i )
-    if( data[i].buffer().handnum == i )
-      delete &data[i].buffer();		// last handle for this buffer
+    if( data[i].buffer().handnum == i )	// last handle for this buffer
+      delete &data[i].buffer();
   data.clear(); first_ = 0; last_visited_ = 0; Block::reset();
   }
