@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cctype>
@@ -238,7 +238,7 @@ int map_to_byte( const int code )
     case 0x017C: return 'z';	// latin small letter z with dot above
     case 0x017D: return 0xB4;	// latin capital letter z with caron
     case 0x017E: return 0xB8;	// latin small letter z with caron
-    case 0x017F: return 's';	// latin small letter long s
+    case 0x017F: return 'f';	// latin small letter long s
     case 0x01CD: return 'A';	// latin capital letter a with caron
     case 0x01CE: return 'a';	// latin small letter a with caron
     case 0x01CF: return 'I';	// latin capital letter i with caron
@@ -249,6 +249,21 @@ int map_to_byte( const int code )
     case 0x01D4: return 'u';	// latin small letter u with caron
     case 0x0232: return 'Y';	// latin capital letter y with macron
     case 0x0233: return 'y';	// latin small letter y with macron
+    case 0x02C6: return '^';	// modifier letter circumflex accent
+    case 0x02DC: return '~';	// small tilde
+    // combining diacritical marks
+    case 0x0300: return '`';	// combining grave accent
+    case 0x0301: return '\'';	// combining acute accent
+    case 0x0302: return '^';	// combining circumflex accent
+    case 0x0303: return '~';	// combining tilde
+    case 0x0304:		// combining macron
+    case 0x0305: return 0xAF;	// combining overline
+    case 0x0308: return '"';	// combining diaeresis
+    case 0x030A: return 0xB0;	// combining ring above
+    case 0x030B:		// combining double acute accent
+    case 0x030E: return '"';	// combining double vertical line above
+    case 0x0331:		// combining macron below
+    case 0x0332: return '_';	// combining low line
     case 0x0374: return '\'';	// greek numeral sign
     case 0x0375: return ',';	// greek lower numeral sign
     case 0x037E: return ';';	// greek question mark
@@ -257,11 +272,11 @@ int map_to_byte( const int code )
     case 0x0386: return 0xC1;	// greek capital letter alpha with tonos
     case 0x0387: return 0xB7;	// greek ano teleia
     case 0x0388: return 0xC9;	// greek capital letter epsilon with tonos
-    case 0x0389: return 0xCD;	// greek capital letter eta with tonos
+    case 0x0389: return 'H';	// greek capital letter eta with tonos
     case 0x038A: return 0xCD;	// greek capital letter iota with tonos
     case 0x038C: return 0xD3;	// greek capital letter omicron with tonos
     case 0x038E: return 0xDD;	// greek capital letter upsilon with tonos
-    case 0x038F: return 0xD3;	// greek capital letter omega with tonos
+    case 0x038F: return 'W';	// greek capital letter omega with tonos
     case 0x0390: return 0xEF;	// greek small letter iota with dialytika tonos
     case 0x0391: return 'A';	// greek capital letter alpha
     case 0x0392: return 'B';	// greek capital letter beta
@@ -269,7 +284,7 @@ int map_to_byte( const int code )
     case 0x0394: return 'D';	// greek capital letter delta
     case 0x0395: return 'E';	// greek capital letter epsilon
     case 0x0396: return 'Z';	// greek capital letter zeta
-    case 0x0397: return 'I';	// greek capital letter eta
+    case 0x0397: return 'H';	// greek capital letter eta
     case 0x0399: return 'I';	// greek capital letter iota
     case 0x039A: return 'K';	// greek capital letter kappa
     case 0x039B: return 'L';	// greek capital letter lambda
@@ -283,12 +298,12 @@ int map_to_byte( const int code )
     case 0x03A4: return 'T';	// greek capital letter tau
     case 0x03A5: return 'Y';	// greek capital letter upsilon
     case 0x03A6: return 'F';	// greek capital letter phi
-    case 0x03A9: return 'O';	// greek capital letter omega
+    case 0x03A9: return 'W';	// greek capital letter omega
     case 0x03AA: return 0xCF;	// greek capital letter iota with dialytika
     case 0x03AB: return 0xBE;	// greek capital letter upsilon with dialytika
     case 0x03AC: return 0xE1;	// greek small letter alpha with tonos
     case 0x03AD: return 0xE9;	// greek small letter epsilon with tonos
-    case 0x03AE: return 0xED;	// greek small letter eta with tonos
+    case 0x03AE: return 'h';	// greek small letter eta with tonos
     case 0x03AF: return 0xED;	// greek small letter iota with tonos
     case 0x03B1: return 'a';	// greek small letter alpha
     case 0x03B2: return 'b';	// greek small letter beta
@@ -296,7 +311,7 @@ int map_to_byte( const int code )
     case 0x03B4: return 'd';	// greek small letter delta
     case 0x03B5: return 'e';	// greek small letter epsilon
     case 0x03B6: return 'z';	// greek small letter zeta
-    case 0x03B7: return 'i';	// greek small letter eta
+    case 0x03B7: return 'h';	// greek small letter eta
     case 0x03B9: return 'i';	// greek small letter iota
     case 0x03BA: return 'k';	// greek small letter kappa
     case 0x03BB: return 'l';	// greek small letter lambda
@@ -311,12 +326,12 @@ int map_to_byte( const int code )
     case 0x03C4: return 't';	// greek small letter tau
     case 0x03C5: return 'y';	// greek small letter upsilon
     case 0x03C6: return 'f';	// greek small letter phi
-    case 0x03C9: return 'o';	// greek small letter omega
+    case 0x03C9: return 'w';	// greek small letter omega
     case 0x03CA: return 0xEF;	// greek small letter iota with dialytika
     case 0x03CB: return 0xFF;	// greek small letter upsilon with dialytika
     case 0x03CC: return 0xF3;	// greek small letter omicron with tonos
     case 0x03CD: return 0xFD;	// greek small letter upsilon with tonos
-    case 0x03CE: return 0xF3;	// greek small letter omega with tonos
+    case 0x03CE: return 'w';	// greek small letter omega with tonos
     case 0x03D0: return 'b';	// greek beta symbol
     case 0x03D2: return 'Y';	// greek upsilon with hook symbol
     case 0x03D3: return 0xDD;	// greek upsilon with acute and hook symbol
@@ -546,6 +561,122 @@ int map_to_byte( const int code )
     case 0x10EE: return 'x';	// georgian letter xan
     case 0x10EF: return 'j';	// georgian letter jhan
     case 0x10F0: return 'h';	// georgian letter hae
+    // psili = oxia = acute, dasia = varia = grave, perispomeni = tilde
+    case 0x1F00: return 0xE1;	// greek small letter alpha with psili
+    case 0x1F01: return 0xE0;	// greek small letter alpha with dasia
+    case 0x1F02: return 0xE2;	// greek small letter alpha with psili and varia
+    case 0x1F03:		// greek small letter alpha with dasia and varia
+    case 0x1F04:		// greek small letter alpha with psili and oxia
+    case 0x1F05: return 0xE4;	// greek small letter alpha with dasia and oxia
+    case 0x1F06:		// greek small letter alpha with psili and perispomeni
+    case 0x1F07: return 0xE3;	// greek small letter alpha with dasia and perispomeni
+    case 0x1F08: return 0xC1;	// greek capital letter alpha with psili
+    case 0x1F09: return 0xC0;	// greek capital letter alpha with dasia
+    case 0x1F0A: return 0xC2;	// greek capital letter alpha with psili and varia
+    case 0x1F0B:		// greek capital letter alpha with dasia and varia
+    case 0x1F0C:		// greek capital letter alpha with psili and oxia
+    case 0x1F0D: return 0xC4;	// greek capital letter alpha with dasia and oxia
+    case 0x1F0E:		// greek capital letter alpha with psili and perispomeni
+    case 0x1F0F: return 0xC3;	// greek capital letter alpha with dasia and perispomeni
+    case 0x1F10: return 0xE9;	// greek small letter epsilon with psili
+    case 0x1F11: return 0xE8;	// greek small letter epsilon with dasia
+    case 0x1F12: return 0xEA;	// greek small letter epsilon with psili and varia
+    case 0x1F13:		// greek small letter epsilon with dasia and varia
+    case 0x1F14:		// greek small letter epsilon with psili and oxia
+    case 0x1F15: return 0xEB;	// greek small letter epsilon with dasia and oxia
+    case 0x1F18: return 0xC9;	// greek capital letter epsilon with psili
+    case 0x1F19: return 0xC8;	// greek capital letter epsilon with dasia
+    case 0x1F1A: return 0xCA;	// greek capital letter epsilon with psili and varia
+    case 0x1F1B:		// greek capital letter epsilon with dasia and varia
+    case 0x1F1C:		// greek capital letter epsilon with psili and oxia
+    case 0x1F1D: return 0xCB;	// greek capital letter epsilon with dasia and oxia
+    case 0x1F20:		// greek small letter eta with psili
+    case 0x1F21:		// greek small letter eta with dasia
+    case 0x1F22:		// greek small letter eta with psili and varia
+    case 0x1F23:		// greek small letter eta with dasia and varia
+    case 0x1F24:		// greek small letter eta with psili and oxia
+    case 0x1F25:		// greek small letter eta with dasia and oxia
+    case 0x1F26:		// greek small letter eta with psili and perispomeni
+    case 0x1F27: return 'h';	// greek small letter eta with dasia and perispomeni
+    case 0x1F28:		// greek capital letter eta with psili
+    case 0x1F29:		// greek capital letter eta with dasia
+    case 0x1F2A:		// greek capital letter eta with psili and varia
+    case 0x1F2B:		// greek capital letter eta with dasia and varia
+    case 0x1F2C:		// greek capital letter eta with psili and oxia
+    case 0x1F2D:		// greek capital letter eta with dasia and oxia
+    case 0x1F2E:		// greek capital letter eta with psili and perispomeni
+    case 0x1F2F: return 'H';	// greek capital letter eta with dasia and perispomeni
+    case 0x1F30: return 0xED;	// greek small letter iota with psili
+    case 0x1F31: return 0xEC;	// greek small letter iota with dasia
+    case 0x1F32: return 0xEE;	// greek small letter iota with psili and varia
+    case 0x1F33:		// greek small letter iota with dasia and varia
+    case 0x1F34:		// greek small letter iota with psili and oxia
+    case 0x1F35: return 0xEF;	// greek small letter iota with dasia and oxia
+    case 0x1F36:		// greek small letter iota with psili and perispomeni
+    case 0x1F37: return 0xEE;	// greek small letter iota with dasia and perispomeni
+    case 0x1F38: return 0xCD;	// greek capital letter iota with psili
+    case 0x1F39: return 0xCC;	// greek capital letter iota with dasia
+    case 0x1F3A: return 0xCE;	// greek capital letter iota with psili and varia
+    case 0x1F3B:		// greek capital letter iota with dasia and varia
+    case 0x1F3C:		// greek capital letter iota with psili and oxia
+    case 0x1F3D: return 0xCF;	// greek capital letter iota with dasia and oxia
+    case 0x1F3E:		// greek capital letter iota with psili and perispomeni
+    case 0x1F3F: return 0xCE;	// greek capital letter iota with dasia and perispomeni
+    case 0x1F40: return 0xF3;	// greek small letter omicron with psili
+    case 0x1F41: return 0xF2;	// greek small letter omicron with dasia
+    case 0x1F42: return 0xF4;	// greek small letter omicron with psili and varia
+    case 0x1F43:		// greek small letter omicron with dasia and varia
+    case 0x1F44:		// greek small letter omicron with psili and oxia
+    case 0x1F45: return 0xF6;	// greek small letter omicron with dasia and oxia
+    case 0x1F48: return 0xD3;	// greek capital letter omicron with psili
+    case 0x1F49: return 0xD2;	// greek capital letter omicron with dasia
+    case 0x1F4A: return 0xD4;	// greek capital letter omicron with psili and varia
+    case 0x1F4B:		// greek capital letter omicron with dasia and varia
+    case 0x1F4C:		// greek capital letter omicron with psili and oxia
+    case 0x1F4D: return 0xD6;	// greek capital letter omicron with dasia and oxia
+    case 0x1F50:		// greek small letter upsilon with psili
+    case 0x1F51: return 0xFD;	// greek small letter upsilon with dasia
+    case 0x1F52:		// greek small letter upsilon with psili and varia
+    case 0x1F53:		// greek small letter upsilon with dasia and varia
+    case 0x1F54:		// greek small letter upsilon with psili and oxia
+    case 0x1F55:		// greek small letter upsilon with dasia and oxia
+    case 0x1F56:		// greek small letter upsilon with psili and perispomeni
+    case 0x1F57: return 0xFF;	// greek small letter upsilon with dasia and perispomeni
+    case 0x1F59: return 0xDD;	// greek capital letter upsilon with dasia
+    case 0x1F5B:		// greek capital letter upsilon with dasia and varia
+    case 0x1F5D:		// greek capital letter upsilon with dasia and oxia
+    case 0x1F5F: return 0xBE;	// greek capital letter upsilon with dasia and perispomeni
+    case 0x1F60:		// greek small letter omega with psili
+    case 0x1F61:		// greek small letter omega with dasia
+    case 0x1F62:		// greek small letter omega with psili and varia
+    case 0x1F63:		// greek small letter omega with dasia and varia
+    case 0x1F64:		// greek small letter omega with psili and oxia
+    case 0x1F65:		// greek small letter omega with dasia and oxia
+    case 0x1F66:		// greek small letter omega with psili and perispomeni
+    case 0x1F67: return 'w';	// greek small letter omega with dasia and perispomeni
+    case 0x1F68:		// greek capital letter omega with psili
+    case 0x1F69:		// greek capital letter omega with dasia
+    case 0x1F6A:		// greek capital letter omega with psili and varia
+    case 0x1F6B:		// greek capital letter omega with dasia and varia
+    case 0x1F6C:		// greek capital letter omega with psili and oxia
+    case 0x1F6D:		// greek capital letter omega with dasia and oxia
+    case 0x1F6E:		// greek capital letter omega with psili and perispomeni
+    case 0x1F6F: return 'W';	// greek capital letter omega with dasia and perispomeni
+    case 0x1F70: return 0xE0;	// greek small letter alpha with varia
+    case 0x1F71: return 0xE1;	// greek small letter alpha with oxia
+    case 0x1F72: return 0xE8;	// greek small letter epsilon with varia
+    case 0x1F73: return 0xE9;	// greek small letter epsilon with oxia
+    case 0x1F74:		// greek small letter eta with varia
+    case 0x1F75: return 'h';	// greek small letter eta with oxia
+    case 0x1F76: return 0xEC;	// greek small letter iota with varia
+    case 0x1F77: return 0xED;	// greek small letter iota with oxia
+    case 0x1F78: return 0xF2;	// greek small letter omicron with varia
+    case 0x1F79: return 0xF3;	// greek small letter omicron with oxia
+    case 0x1F7A:		// greek small letter upsilon with varia
+    case 0x1F7B: return 0xFD;	// greek small letter upsilon with oxia
+    case 0x1F7C:		// greek small letter omega with varia
+    case 0x1F7D: return 'w';	// greek small letter omega with oxia
+    case 0x2020: return '*';	// dagger
     case 0x2022: return 0xB7;	// bullet
     case 0x2024: return '.';
     case 0x2027: return 0xB7;	// hyphenation point
@@ -604,6 +735,9 @@ int map_to_byte( const int code )
     case 0x209B: return 's';	// latin subscript small letter s
     case 0x209C: return 't';	// latin subscript small letter t
     case 0x20AC: return 0xA4;	// euro sign
+    case 0x2191: return '^';	// upwards arrow
+    case 0x2193: return 'V';	// downwards arrow
+    case 0x2211: return 'S';	// N-ary summation
     case 0x2212: return '-';	// minus sign
     case 0x2213: return 0xB1;	// minus-or-plus sign
     case 0x2215: return '/';	// division slash
@@ -614,25 +748,30 @@ int map_to_byte( const int code )
     case 0x2223: return '|';	// divides
     case 0x2236: return ':';	// ratio
     case 0x223C: return '~';	// tilde operator
+    case 0x2261: return '=';	// identical to
     case 0x227A: return '<';	// precedes
     case 0x227B: return '>';	// succeeds
     case 0x22B0: return '<';	// precedes under relation
     case 0x22B1: return '>';	// succeeds under relation
     case 0x22C5: return 0xB7;	// dot operator
+    case 0x23AF: return '-';	// horizontal line extension
     case 0x27E8: return '<';	// mathematical left angle bracket
     case 0x27E9: return '>';	// mathematical right angle bracket
     case 0x3001: return ',';	// ideographic comma
     case 0x3002: return '.';	// ideographic full stop
+    case 0xFFFD: return '?';	// replacement character (unknown character)
     }
   if( code < 256 ) return code;
 //  if( code >= 0x02B0 && code <= 0x02FF ) return 256;	// spacing modifier letters
 //  if( code >= 0x0300 && code <= 0x036F ) return 256;	// combining diacritical marks
   if( ( code >= 0x2000 && code <= 0x200B ) ||
       code == 0x202F || code == 0x205F || code == 0x3000 ) return ' ';
+  if( code == 0x200C || code == 0x200D ) return 256;	// zero width (non-)joiner
   if( ( code >= 0x2010 && code <= 0x2013 ) || code == 0x2043 ) return '-';
   if( ( code >= 0x2018 && code <= 0x201B ) || code == 0x2032 ) return '\'';
   if( ( code >= 0x201C && code <= 0x201F ) || code == 0x2033 || code == 0x2036 )
     return '"';
+  if( code >= 0x239B && code <= 0x23AD ) return '|';
   return -1;
   }
 
@@ -690,8 +829,11 @@ const char * map_to_string( const int code )
     case 0x10EB: return "dz";	// georgian letter jil
     case 0x10EC: return "ts'";	// georgian letter cil
     case 0x10ED: return "ch'";	// georgian letter char
-    case 0x2014:
-    case 0x2015: return "--";
+    case 0x1E9E: return "SS";	// latin capital letter sharp s (german)
+    case 0x2014:		// em dash
+    case 0x2015: return "--";	// horizontal bar
+    case 0x2016: return "||";	// double vertical line (norm of matrix)
+    case 0x2017: return "__";	// double low line
     case 0x2025: return "..";
     case 0x2026: return "...";
     case 0x2034: return "'''";
@@ -702,11 +844,28 @@ const char * map_to_string( const int code )
     case 0x2049: return "!?";
     case 0x2057: return "''''";
     case 0x2122: return "TM";	// trade mark sign
+    case 0x2190: return "<-";	// leftwards arrow
+    case 0x2192: return "->";	// rightwards arrow
+    case 0x2194: return "<->";	// left right arrow
+    case 0x221A: return "\\/";	// square root
+    case 0x221E: return "oo";	// infinity
     case 0x2237: return "::";	// proportion
+    case 0x2262: return "!=";	// not identical to
     case 0x2264: return "<=";	// less-than or equal to
     case 0x2265: return ">=";	// greater-than or equal to
     case 0x227C: return "<=";	// precedes or equal to
     case 0x227D: return ">=";	// succeeds or equal to
+    case 0x2639: return ":-(";	// white frowning face
+    case 0x263A: return ":-)";	// white smiling face
+    case 0x263B: return ":-)";	// black smiling face
+    // alphabetic presentation forms
+    case 0xFB00: return "ff";	// latin small ligature ff
+    case 0xFB01: return "fi";	// latin small ligature fi
+    case 0xFB02: return "fl";	// latin small ligature fl
+    case 0xFB03: return "ffi";	// latin small ligature ffi
+    case 0xFB04: return "ffl";	// latin small ligature ffl
+    case 0xFB05: return "ft";	// latin small ligature long s t
+    case 0xFB06: return "st";	// latin small ligature st
     }
   return 0;
   }
@@ -967,10 +1126,10 @@ bool Encoding::utf8_encode( const std::string & in, std::string & out,
   }
 
 
-// 'seq' contains an UTF-8 (possibly) multibyte character sequence.
-// Returns the corresponding code and, in *lenp, the characters read.
-// Returns -1 if error.
-//
+/* 'seq' contains an UTF-8 (possibly) multibyte character sequence.
+   Returns the corresponding code and, in *lenp, the number of bytes read.
+   Returns -1 if error.
+*/
 int Encoding::utf8_to_ucs( const std::string & seq, const unsigned i,
                            int * const lenp )
   {
