@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cctype>
@@ -125,10 +125,10 @@ enum Code {
 } // end namespace ISO_8859
 
 
-// 'seq[i]' begins a escape sequence (the characters following a '\').
-// Returns the corresponding code and, in '*lenp', the characters read.
-// Returns -1 if error.
-//
+/* 'seq[i]' begins a escape sequence (the characters following a '\').
+   Returns the corresponding code and, in *lenp, the number of bytes read.
+   Returns -1 if error.
+*/
 int ISO_8859::escape( const std::string & seq, const unsigned i, int * lenp )
   {
   if( i >= seq.size() ) return -1;
@@ -170,7 +170,7 @@ void ISO_8859::escapize( std::string & s )
   for( unsigned i = 0; i < s.size(); ++i )
     {
     if( !ISO_8859::iscntrl( s[i] ) && s[i] != '\\' ) continue;
-    s.insert( i, 1, '\\' ); ++i;
+    s.insert( s.begin() + i, '\\' ); ++i;
     switch( s[i] )
       {
       case '\\': break;
@@ -183,9 +183,9 @@ void ISO_8859::escapize( std::string & s )
       case '\v': s[i] = 'v'; break;
       default:
         {
-        s.insert( i, 1, 'x' ); ++i;
+        s.insert( s.begin() + i, 'x' ); ++i;
         const unsigned char ch = s[i];
-        s.insert( i, 1, ISO_8859::xdigit( ch >> 4 ) ); ++i;
+        s.insert( s.begin() + i, ISO_8859::xdigit( ch >> 4 ) ); ++i;
         s[i] = ISO_8859::xdigit( ch & 0x0F );
         }
       }
@@ -216,6 +216,8 @@ bool ISO_8859::islower( const unsigned char ch )
     case SSCARON:
     case SZCARON:
     case SLIGOE:
+//  SSSHARP commented out because ISO_8859::toupper( SSSHARP ) does nothing
+//  case SSSHARP:
     case SAGRAVE:
     case SAACUTE:
     case SACIRCU:
