@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -101,12 +101,14 @@ void append_part_or_go_up( std::string & name, const std::string & new_part )
   {
   if( new_part == ".." )
     {
+    int i = name.size();	// name is either empty or ends in '/'
     if( name == "/" ) return;
-    if( name == "//" ||
-        ( name.size() >= 4 && name.compare( name.size() - 4, 4, "/../" ) != 0 ) ||
-        ( name.size() == 3 && name != "../" ) )
+    if( i == 2 || ( i == 3 && name != "../" ) ||
+        ( i >= 4 && name.compare( i - 4, 4, "/../" ) != 0 ) )
       {
-      int i = name.size() - 1; while( i > 0 && name[i-1] != '/' ) --i;
+      while( i > 1 && name[i-1] == '/' ) --i;	// remove trailing '/'
+      while( i > 0 && name[i-1] != '/' ) --i;	// remove dir
+      while( i > 1 && name[i-2] == '/' ) --i;	// leave one '/' if any exists
       name.resize( i );
       return;
       }
@@ -168,7 +170,7 @@ int Menu::bufhandle_menu( int current, const int abort_key )
 
   Screen::save_lines_and_cursor( line, lines + 1 );
 
-  Screen::out_line( "Select buffer handle, F1-F10 or first char (^C to abort)",
+  Screen::out_line( "Select buffer handle, F1-F10, or first char (^C to abort)",
                     Screen::height() - 1 );
   current = Bufhandle_vector::absolute_index( current );
   int key = 0;
@@ -218,11 +220,12 @@ int Menu::bufhandle_menu( int current, const int abort_key )
   }
 
 
-// 'name' contains the file name to be modified (may be empty).
-// Returns false if aborted.
-//
+/* 'name' contains the file name to be modified (may be empty).
+   Returns false if aborted.
+*/
 bool Menu::file_menu( const std::string & prompt, std::string & name )
   {
+  const std::string name_orig( name );
   std::string dirname, basename;
   std::vector< std::string > name_vector;
 
@@ -231,12 +234,12 @@ bool Menu::file_menu( const std::string & prompt, std::string & name )
   basename += '*';
 
   DIR * const dirp = opendir( dirname.size() ? dirname.c_str() : "./" );
-  if( !dirp ) return true;
+  if( !dirp && ( errno == ENOTDIR || errno == ENOENT ) ) return true;
   bool dd = false;
   const std::string dotdot( ".." );		// put ".." always first
-  if( Regex::match_filename( basename, dotdot ) )
+  if( !dirp || Regex::match_filename( basename, dotdot ) )
     { name_vector.push_back( dotdot ); dd = true; }
-  while( true )
+  if( dirp ) while( true )
     {
     const struct dirent * const entryp = readdir( dirp );
     if( !entryp ) { closedir( dirp ); break; }
@@ -251,11 +254,13 @@ bool Menu::file_menu( const std::string & prompt, std::string & name )
     std::sort( name_vector.begin() + dd, name_vector.end() );
     }
   else if( !entries ) return true;
+/*  else if( entries == 1 && name_vector[0] == ".." )
+    name = dirname + name_vector[0];*/
 
-  if( entries == 1 ||
-    ( entries == 2 && name_vector[0] == ".." && name_vector[1][0] != '.' ) )
-    { name = dirname + name_vector.back(); append_slash_if_dir( name );
-      return true; }
+  if( ( entries == 1 && ( name_vector[0] != ".." || dirname.empty() ) ) ||
+      ( entries == 2 && name_vector[0] == ".." && name_vector[1][0] != '.' ) )
+    { std::string s( dirname ); append_part_or_go_up( s, name_vector.back() );
+      if( s != name_orig ) { name = s; return true; } }		// completion
 
   int max_width = 0;
   for( int i = 0; i < entries; ++i )
@@ -323,11 +328,11 @@ bool Menu::file_menu( const std::string & prompt, std::string & name )
   }
 
 
-// 'name' contains the default result (may be empty), and
-// stores the result (if any).
-// Returns the size of the result string, 0 if none, -1 if aborted.
-// 'name' is cleared if aborted.
-//
+/* 'name' contains the default result (may be empty), and stores the result
+   (if any).
+   Returns the size of the result string, 0 if none, -1 if aborted.
+   'name' is cleared if aborted.
+*/
 int Menu::get_filename( const std::string & prompt, std::string & name,
                         const int abort_key )
   {
