@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cstdio>
@@ -36,20 +36,21 @@ const char * help_line[] = {
   " @bGO TO@b                  @bSEARCH@b                      @bMISC@b",
   " ^[U Beg of file        ^F  Find                    Ins  Insert/Overwrite",
   " ^[V End of file        ^G  Find next               Tab  File name completion",
-  " ^[L Line               ^W  Find word               ^B   Reformat paragraph",
-  " ^[C Column             ^OF Toggle direction        ^P   Control char",
-  " ^[O Offset             ^QU update (C)              ^Y   Delete line",
-  " ^[B Beg of block                                   ^OC  Center line",
-  " ^[K End of block       @bINFO@b                        ^ON  Change buffer name",
-  " ^[F Match pair ->      ^A   Show help (^H)         ^OR  Repaint screen",
-  " ^[G Match pair <-      ^[I  Show character info    ^OS  Split window",
-  " ^[D Center cursor      ^[T  Show UTF-8 code        ^Q@bF2@b Save named buffers",
-  "                        ^Sn  Show LE multibyte",
-  " @bSCROLL@b                 ^S@bFn@b Show BE multibyte      @bBOOKMARKS@b",
-  " ^[W Up                 ^SC  Show character code    ^[ 0-9 Goto",
-  " ^[Z Down               ^SG  Show global status     ^K 0-9 Set",
-  " ^[A Left               ^SP  Show line position     ^KE    Extend",
-  " ^[S Right              ^SV  Show version",
+  " ^[L Line               ^W  Find word               ^A 1-90AB Alternative @bFn@b",
+  " ^[C Column             ^OF Toggle direction        ^B   Reformat paragraph",
+  " ^[O Offset             ^QU update (C)              ^P   Control char",
+  " ^[B Beg of block                                   ^Y   Delete line",
+  " ^[K End of block       @bINFO@b                        ^OC  Center line",
+  " ^[F Match pair ->      ^H   Show help              ^ON  Change buffer name",
+  " ^[G Match pair <-      ^[I  Show character info    ^OR  Repaint screen",
+  " ^[D Center cursor      ^[T  Show UTF-8 code        ^OS  Split window",
+  "                        ^Sn  Show LE multibyte      ^Q@bF2@b Save named buffers",
+  " @bSCROLL@b                 ^S@bFn@b Show BE multibyte",
+  " ^[W Up                 ^SB  Show clock blinking    @bBOOKMARKS@b",
+  " ^[Z Down               ^SC  Show character code    ^[ 0-9 Goto",
+  " ^[A Left               ^SG  Show global status     ^K 0-9 Set",
+  " ^[S Right              ^SP  Show line position     ^KE    Extend",
+  "                        ^SV  Show version",
   "",
   " Help Screen: Block Commands     PAGE_DN next/PAGE_UP prev",
   " ",
@@ -88,18 +89,18 @@ const char * help_line[] = {
   "   \\?      matches any single char       \\^  \\$  matches beginning/end of line",
   "   \\*      matches 0 or more chars       \\<  \\>  matches beginning/end of word",
   "   \\[..]   matches one in a set          \\c      matches balanced C expression",
-  "   \\[^..]  matches one not in a set      \\+<ch>  matches 0 or more of <ch>",
+  "   \\[^..]  matches one not in a set      \\T      matches trailing whitespace",
   "   \\s      matches 0 or more whitespace characters",
   "   \\S      matches the longest possible non-empty sequence of whitespace",
   "   \\w      matches any word-constituent character",
   "   \\W      matches any character that is not a word-constituent",
+  "   \\+<ch>  matches 0 or more of <ch> (also \\[..], \\w, or \\W)",
   " ",
   " Notes",
   "     Ranges of characters may be specified in sets with 'A-Z'",
   "     '-' may be specified in sets by placing it first or last",
   "     ']' may be specified in sets by placing it first",
-  "     \\+\\[..] matches 0 or more chars of those in set",
-  "     \\+\\w    matches 0 or more word-constituent characters",
+  "     \\+ may precede itself any number of times, matching N-1 chars",
   "",
   " Help Screen: Replace            PAGE_DN next/PAGE_UP prev",
   " ",
@@ -228,8 +229,7 @@ void show_help_screen( const int current )
 } // end namespace Menu
 
 
-void Menu::help_menu( const int abort_key1, const int abort_key2,
-                      const int abort_key3 )
+void Menu::help_menu( const int abort_key1, const int abort_key2 )
   {
   static int current = 0;
   int old = -1;				// make old != current
@@ -244,8 +244,7 @@ void Menu::help_menu( const int abort_key1, const int abort_key2,
       Screen::move_to( Point( 0, Screen::width() - 1 ) );
       }
     const int key = Screen::wait_kbhit( 1 );	// show clock on line 0 only
-    if( key == 3 || key == abort_key1 || key == abort_key2 ||
-        key == abort_key3 ) break;				// ^C
+    if( key == 3 || key == abort_key1 || key == abort_key2 ) break;	// ^C
     switch( Screen::convert_key( key ) )
       {
       case Screen::key_down :
