@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cstdio>
@@ -35,10 +35,10 @@ int entries;		// number of entries in menu
 int height_;		// height in lines of menu
 
 
-const char * autoindent_status( const Buffer & buffer )
+const char * auto_indent_status( const Buffer & buffer )
   { return buffer.options.auto_indent ? "ON" : "OFF"; }
 
-bool autoindent_change( Buffer & buffer )
+bool auto_indent_change( Buffer & buffer )
   { buffer.options.auto_indent = !buffer.options.auto_indent; return true; }
 
 
@@ -86,7 +86,7 @@ bool ignore_case_change( Buffer & )
 
 const char * indent_step_status( const Buffer & )
   {
-  static char buf[8];
+  static char buf[12];		// 12 keeps g++ quiet
   snprintf( buf, sizeof buf, "%d", RC::editor_options().indent_step() );
   return buf;
   }
@@ -109,7 +109,7 @@ bool indent_step_change( Buffer & )
 
 const char * keep_lines_status( const Buffer & )
   {
-  static char buf[8];
+  static char buf[12];		// 12 keeps g++ quiet
   snprintf( buf, sizeof buf, "%d", RC::editor_options().keep_lines() );
   return buf;
   }
@@ -132,7 +132,7 @@ bool keep_lines_change( Buffer & )
 
 const char * lmargin_status( const Buffer & buffer )
   {
-  static char buf[8];
+  static char buf[12];		// 12 keeps g++ quiet
   snprintf( buf, sizeof buf, "%d", buffer.options.lmargin() + 1 );
   return buf;
   }
@@ -155,7 +155,7 @@ bool lmargin_change( Buffer & buffer )
 
 const char * max_windows_status( const Buffer & )
   {
-  static char buf[8];
+  static char buf[12];		// 12 keeps g++ quiet
   snprintf( buf, sizeof buf, "%d", RC::editor_options().max_windows() );
   return buf;
   }
@@ -185,7 +185,7 @@ bool read_only_change( Buffer & buffer )
 
 const char * rmargin_status( const Buffer & buffer )
   {
-  static char buf[8];
+  static char buf[12];		// 12 keeps g++ quiet
   snprintf( buf, sizeof buf, "%d", buffer.options.rmargin() + 1 );
   return buf;
   }
@@ -239,6 +239,13 @@ bool rectangle_mode_change( Buffer & )
   }
 
 
+const char * two_spaces_status( const Buffer & buffer )
+  { return buffer.options.two_spaces ? "ON" : "OFF"; }
+
+bool two_spaces_change( Buffer & buffer )
+  { buffer.options.two_spaces = !buffer.options.two_spaces; return true; }
+
+
 struct Entry
   {
   char ch;			// Menu letter
@@ -248,7 +255,7 @@ struct Entry
   };
 
 Entry table[] = {
-  { 'A', "Autoindent ", &autoindent_status, &autoindent_change },
+  { 'A', "Auto indent ", &auto_indent_status, &auto_indent_change },
   { 'B', "Create backups ", &backup_status, &backup_change },
   { 'C', "CR-LF ended lines ", &crlf_status, &crlf_change },
   { 'E', "Exit ask ", &exit_ask_status, &exit_ask_change },
@@ -266,6 +273,7 @@ Entry table[] = {
   { 'U', "Auto unmark ", &auto_unmark_status, &auto_unmark_change },
   { 'W', "Word wrap ", &word_wrap_status, &word_wrap_change },
   { 'X', "Rectangle mode ", &rectangle_mode_status, &rectangle_mode_change },
+  { '2', "Two spaces ", &two_spaces_status, &two_spaces_change },
   { 0, 0, 0, 0 } };
 
 
