@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -90,10 +90,10 @@ enum Optcode
   max_windows = 'm', orphan_extra= '1', rectangle_mode = 'x',
   search_wrap = 's', smart_home = 'H',
   auto_indent = 'a', lmargin = 'l', rmargin = 'r', overwrite = 'O',
-  read_only = 'o', word_wrap = 'w',
+  read_only = 'o', two_spaces = '2', word_wrap = 'w',
   no_auto_unmark = 256, /*no_beep,*/ no_exit_ask, no_ignore_case,
   no_rectangle_mode, no_search_wrap, no_smart_home,
-  no_auto_indent, no_overwrite, no_read_only, no_word_wrap
+  no_auto_indent, no_overwrite, no_read_only, no_two_spaces, no_word_wrap
   };
 
 const Arg_parser::Option options[] =
@@ -128,6 +128,8 @@ const Arg_parser::Option options[] =
   { no_overwrite,      "no-overwrite",   Arg_parser::no  },
   { read_only,         "read-only",      Arg_parser::no  },
   { no_read_only,      "no-read-only",   Arg_parser::no  },
+  { two_spaces,        "two-spaces",     Arg_parser::no  },
+  { no_two_spaces,     "no-two-spaces",  Arg_parser::no  },
   { word_wrap,         "word-wrap",      Arg_parser::no  },
   { no_word_wrap,      "no-word-wrap",   Arg_parser::no  },
   { 0,                 0,                Arg_parser::no  } };
@@ -158,6 +160,8 @@ bool is_editor_option( const int code )
     case no_overwrite  :
     case read_only     :
     case no_read_only  :
+    case two_spaces    :
+    case no_two_spaces :
     case word_wrap     :
     case no_word_wrap  : return false;
     }
@@ -213,6 +217,8 @@ bool set_option_from_code( const int code, const std::string & arg,
     case no_overwrite:
     case read_only:
     case no_read_only:
+    case two_spaces:
+    case no_two_spaces:
     case word_wrap:
     case no_word_wrap:      file_options.add_option( code ); break;
     case lmargin:
@@ -270,6 +276,8 @@ void apply_file_options( Buffer & buffer, const File_options & file_options )
       case no_overwrite:   buffer.options.overwrite = false; break;
       case read_only:      buffer.options.read_only = true; break;
       case no_read_only:   buffer.options.read_only = false; break;
+      case two_spaces:     buffer.options.two_spaces = true; break;
+      case no_two_spaces:  buffer.options.two_spaces = false; break;
       case word_wrap:      buffer.options.word_wrap = true; break;
       case no_word_wrap:   buffer.options.word_wrap = false; break;
       }
@@ -298,27 +306,30 @@ bool unwanted_name( const std::string & name )
   {
   const char * const ext1[] = { "a", "o", "z", "Z", 0 };
   const char * const ext2[] = { "gz", "lz", "mo", "po", "ps", "so", "xz", 0 };
-  const char * const ext3[] = { "bz2", "deb", "dll", "dvi", "gif", "gmo",
-    "ico", "jar", "jpg", "pbm", "pdf", "pgm", "png", "ppm", "rpm", "svg",
-    "tar", "tgz", "tif", "tlz", "txz", "wav", 0 };
-  const char * const ext4[] = { "rpms", "svgz", 0 };
-  const char * const * ext[4] = { ext1, ext2, ext3, ext4 };
+  const char * const ext3[] = { "bmp", "bz2", "deb", "dll", "dvi", "elf",
+    "gif", "gmo", "ico", "jar", "jpg", "obj", "pbm", "pdf", "pgm", "png",
+    "ppm", "rpm", "svg", "tar", "tgz", "tif", "tlz", "txz", "wav", "xcf",
+    "zip", 0 };
+  const char * const ext4[] = { "info", "jpeg", "rpms", "svgz", 0 };
+  const char * const ext5[] = { "class", 0 };
+  const char * const * ext[5] = { ext1, ext2, ext3, ext4, ext5 };
 
   if( name == "." || name == ".." || name.empty() ||
       name[name.size()-1] == '~' ) return true;
-  for( unsigned i = 0; i < 4; ++i )
+  for( unsigned i = 0; i < 5; ++i )
     {
     if( name.size() <= i + 2 ) break;
     if( name[name.size()-i-2] != '.' ) continue;
     for( const char * const * p = ext[i]; *p; ++p )
       if( name.compare( name.size() - i - 1, i + 1, *p ) == 0 ) return true;
+    break;			// compare only one extension size
     }
   return false;
   }
 
 
 // Depth first. Files first in each directory. Load regular files only.
-// Ignore backups '*~', unwanted names and symbolic links.
+// Ignore backups '*~', unwanted names, and symbolic links.
 bool add_handles_recursively( std::string name, const int line, const int col,
                               const File_options & file_options,
                               bool & first_post, bool & read_error_found )
@@ -408,7 +419,7 @@ void RC::apply_all_file_options( Buffer & buffer )
   }
 
 
-     // Returns true if first non whitespace character is '+' or '-'.
+// Returns true if first non-whitespace character is '+' or '-'.
 bool RC::issigned( const std::string & s )
   {
   unsigned i = 0;
@@ -417,7 +428,7 @@ bool RC::issigned( const std::string & s )
   }
 
 
-    // Returns the number of chars read, or 0 if error.
+// Returns the number of bytes read, or 0 if error.
 int RC::parse_int( const std::string & s, int & result )
   {
   if( s.empty() ) return 0;
@@ -432,7 +443,7 @@ int RC::parse_int( const std::string & s, int & result )
   }
 
 
-    // Returns the number of chars read, or 0 if error.
+// Returns the number of bytes read, or 0 if error.
 int RC::parse_relative_int( const std::string & s, int & result )
   {
   int tmp = 0;
@@ -453,7 +464,7 @@ int RC::parse_relative_int( const std::string & s, int & result )
   }
 
 
-    // Returns 0 for success, 1 for file not found, 2 for syntax error.
+// Returns 0 for success, 1 for file not found, 2 for syntax error.
 int RC::process_rcfile( const std::string & name )
   {
   FILE * const f = std::fopen( name.c_str(), "r" );
@@ -523,7 +534,7 @@ int RC::process_rcfile( const std::string & name )
   }
 
 
-    // Returns 0 if success, 1 if invalid option, 2 if a file can't be read.
+// Returns 0 if success, 1 if invalid option, 2 if a file can't be read.
 int RC::process_options()
   {
   std::fputs( "Processing options... ", stderr );
@@ -541,8 +552,8 @@ int RC::process_options()
 
   int line = 0, col = 0;
   bool first_post = true;
-  bool non_regular_found = false;
   bool read_error_found = false;
+  bool unsupported_found = false;
   bool stdin_used = false;
   while( argind < parserp->arguments() )  // Process files and file options
     {
@@ -576,8 +587,8 @@ int RC::process_options()
       }
     bool can_read = false;
     const bool is_stdin = ( name == "-" );
-    if( is_stdin && ( stdin_used || isatty( fileno( stdin ) ) ) );
-      // ignore stdin if is repeated or is a terminal
+    if( is_stdin && ( stdin_used || isatty( fileno( stdin ) ) ) )
+      ;		// ignore stdin if is repeated or is a terminal
     else if( Menu::is_regular( name, &can_read ) || can_read )
       {
       try {
@@ -596,8 +607,8 @@ int RC::process_options()
                                        first_post, read_error_found ) )
       {
       if( first_post ) { first_post = false; std::fputc( '\n', stderr ); }
-      std::fprintf( stderr, "warning: '%s' is not a regular file\n", name.c_str() );
-      non_regular_found = true;
+      std::fprintf( stderr, "warning: '%s' is not a supported file type\n", name.c_str() );
+      unsupported_found = true;
       }
     if( is_stdin ) stdin_used = true;
     }
@@ -612,12 +623,12 @@ int RC::process_options()
 
   std::fputs( "done\n", stderr );
   delete parserp;
-  if( non_regular_found || read_error_found )
+  if( read_error_found || unsupported_found )
     {
-    if( non_regular_found )
-      std::fputs( "warning: Non-regular files will be ignored.\n", stderr );
     if( read_error_found )
       std::fputs( "warning: One or more files couldn't be read.\n", stderr );
+    if( unsupported_found )
+      std::fputs( "warning: File types not supported will be ignored.\n", stderr );
     return 2;
     }
   return 0;
