@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cctype>
@@ -129,8 +129,7 @@ inline int case_plus( const Basic_buffer & buffer, Point & p,
           if( tmp == 0 ) break; else if( tmp != 1 ) return 2;
           }
         const int ch2 = buffer.pgetc( p );
-        if( ch2 < 0 || ( chr == 'w' ) != ISO_8859::isalnum_( ch2 ) )
-          return 1;
+        if( ch2 < 0 || ( chr == 'w' ) != ISO_8859::isalnum_( ch2 ) ) return 1;
         piece += ch2;
         }
       pieces.insert( pieces.begin() + ps, piece ); i = regex.size();
@@ -158,10 +157,11 @@ inline int case_plus( const Basic_buffer & buffer, Point & p,
   }
 
 
-// Returns 0 if substring of 'buffer' beginning at 'p' matches 'regex'.
-// If found, sets 'p' after found substring and returns in 'pieces' the
-// text matching every \*, \? etc from the regular expresion.
-//
+/* Returns 0 if a substring of 'buffer' beginning at 'p' matches 'regex[i]'.
+   If match, sets 'p' after the substring found and returns in 'pieces' the
+   text matching every '\*', '\?', etc, from the regular expresion.
+   Returns 1 if mismatch, 2 if regex syntax error.
+*/
 int match_regex( const Basic_buffer & buffer, Point & p,
                  const std::string & regex, unsigned i,
                  std::vector< std::string > & pieces, const bool icase )
@@ -272,6 +272,19 @@ int match_regex( const Basic_buffer & buffer, Point & p,
                 if( retval == 0 )
                   { pieces.insert( pieces.begin() + ps, piece ); i = regex.size(); }
                 } break;
+      case 'T': {			// Find longest matching sequence
+                std::string piece;
+                while( true )
+                  {
+                  const int ch2 = buffer[p];
+                  if( ch2 == ' ' || ch2 == '\t' || ch2 == 0xA0 )
+                    { piece += ch2; buffer.pnext( p ); continue; }
+                  if( ( ch2 < 0 || ch2 == '\n' ) && piece.size() )
+                    pieces.push_back( piece );
+                  else retval = 1;
+                  break;
+                  }
+                } break;
       case 'W':
       case 'w': {
                 const int ch2 = buffer.pgetc( p );
@@ -300,12 +313,12 @@ int match_regex( const Basic_buffer & buffer, Point & p,
 } // end namespace Regex
 
 
-// Searches 'buffer' from 'p1' ('p1-1' if backward) for 'regex'.
-// If found, sets 'p1' at the first char of, and 'p2' after the
-// found substring and returns in 'pieces' the text matching every \*,
-// \? etc from the regular expresion.
-// Returns 0 if match, 1 if no match, 2 if trouble.
-//
+/* Searches 'buffer' from 'p1' ('p1-1' if backward) for 'regex'.
+   If found, sets 'p1' at the first char of, and 'p2' after the end of, the
+   substring found, and returns in 'pieces' the text matching every '\*',
+   '\?', etc, from the regular expresion.
+   Returns 0 if match, 1 if no match, 2 if trouble.
+*/
 int Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
                  const std::string & regex, std::vector< std::string > & pieces,
                  const bool icase, const bool backward )
@@ -354,11 +367,11 @@ int Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
   }
 
 
-// Deletes block at [p1,p2) and replaces it with the text produced from
-// 'regex' and 'pieces'.
-// If there is no error, returns true and sets 'p2' after the newly
-// inserted text.
-//
+/* Deletes block at [p1,p2) and replaces it with the text produced from
+   'regex' and 'pieces'.
+   If there is no error, returns true and sets 'p2' after the newly
+   inserted text.
+*/
 bool Regex::replace( Buffer & buffer, const Point & p1, Point & p2,
                      const std::string & regex,
                      const std::vector< std::string > & pieces )
@@ -413,40 +426,34 @@ bool Regex::replace( Buffer & buffer, const Point & p1, Point & p2,
 bool Regex::match_filename( const std::string & regex, const std::string & name,
                             unsigned i1, unsigned i2 )
   {
-  if( i1 >= regex.size() || i2 >= name.size() ) return false;
-  bool fail = false;
-
-  for( ; i1 < regex.size() && !fail; ++i1 )
+  for( ; i1 < regex.size(); ++i1 )
     {
     unsigned char ch1 = regex[i1];
+    if( ch1 == '*' )
+      {
+      if( i1 + 1 >= regex.size() ) return true;		// tail match
+      for( ; i2 < name.size(); ++i2 )
+        if( match_filename( regex, name, i1 + 1, i2 ) ) return true;
+      return false;
+      }
+    if( i2 >= name.size() ) return false;
     switch( ch1 )
       {
-      case '?': if( ++i2 > name.size() ) fail = true; break;
-      case '*': if( i1 + 1 >= regex.size() ) { i2 = name.size(); break; }
-                if( regex[i1+1] == '*' ) break;
-                for( ; i2 < name.size(); ++i2 )
-                  if( match_filename( regex, name, i1 + 1, i2 ) )
-                    break;
-                if( i2 >= name.size() ) { fail = true; break; }
-                i1 = regex.size(); i2 = name.size(); break;
+      case '?': ++i2; break;
       case '[': {
-                if( i2 >= name.size() ) { fail = true; break; }
-                unsigned char ch2 = name[i2++];
                 Set set( regex, i1 );		// Set of characters to match
-                if( set.includes( ch2 ) ) i1 += set.parsed_len() - 1;
-                else fail = true;
+                if( !set.includes( name[i2++] ) ) return false;
+                i1 += set.parsed_len() - 1;
                 } break;
       default: if( ch1 == '\\' )
                  {
                  int len, cht = ISO_8859::escape( regex, i1 + 1, &len );
-                 if( cht < 0 ) { fail = true; break; }
+                 if( cht < 0 ) return false;
                  ch1 = cht; i1 += len;
                  }
-               if( i2 >= name.size() || ch1 != (unsigned char)name[i2++] )
-                 fail = true;
+               if( ch1 != (unsigned char)name[i2++] ) return false;
                break;
       }
     }
-  if( !fail && i2 < name.size() ) fail = true;
-  return !fail;
+  return ( i2 >= name.size() );
   }
