@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -62,12 +62,13 @@ public:
 
 char * line_buf_;
 int line_buf_size;
-int height_, width_;
+int height_ = 0, width_ = 0;	// if > 0, init has been called
 int clock_limit;		//   0: disable clock
 				//   1: showing help
 				// > 1: first line used by menu
 enum { clock_string_size = 5 };
 char clock_string_[clock_string_size+1] = " 0:00";
+char clock_char = ':';		// ' ' or ':' to enable/disable blinking
 
 std::vector< Lines_and_cursor > lines_and_cursor_vector;
 
@@ -112,11 +113,11 @@ void out_raw_string( const int line, const int col, const std::string & s,
   }
 
 
-// '@b' sets-resets bold mode
-// '@i' sets-resets inverse video mode
-// '@u' sets-resets underline mode
-// '@@' shows a '@'
-//
+/* '@b' sets-resets bold mode
+   '@i' sets-resets inverse video mode
+   '@u' sets-resets underline mode
+   '@@' shows a '@'
+*/
 void out_string( const int line, const int col, const std::string & s,
                  const int len, int pos = 0 )
   {
@@ -183,6 +184,32 @@ const char * alt_key_loop( int key )
   }
 
 
+int alternative_key( int key )
+  {
+  Window & w = Window_vector::curwin();
+  w.show_status_line( ISO_8859::control_name( key ) );
+  do key = wait_kbhit(); while( key < 0 );
+  w.show_status_line();
+  key = ISO_8859::decontrolize( key );
+  switch( key )
+    {
+    case '1': return KEY_F(1);
+    case '2': return KEY_F(2);
+    case '3': return KEY_F(3);
+    case '4': return KEY_F(4);
+    case '5': return KEY_F(5);
+    case '6': return KEY_F(6);
+    case '7': return KEY_F(7);
+    case '8': return KEY_F(8);
+    case '9': return KEY_F(9);
+    case '0': return KEY_F(10);
+    case 'A': return KEY_F(11);
+    case 'B': return KEY_F(12);
+    }
+  return -1;
+  }
+
+
 const char * control_k_loop( int key )
   {
   const char * msg = 0;
@@ -258,11 +285,13 @@ int control_q_loop( int key, const char ** const msgp )
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
+  if( key == 1 ) key = alternative_key( key );			// ^A
   key = ISO_8859::decontrolize( key );
   last_key.shift( key );
   switch( key )
     {
-    case KEY_F(2): *msgp = Bufhandle_vector::save_all_named(); break;
+    case KEY_F(2):
+    case '2': *msgp = Bufhandle_vector::save_all_named(); break;
     case 'C': if( Window_vector::close_and_exit( true ) == 0 ) return 0; break;
     case 'U': *msgp = Window_vector::copyright_update(); break;
     case 'X': if( Window_vector::close_and_exit( false ) == 0 ) return 0; break;
@@ -278,6 +307,7 @@ const char * control_s_loop( int key, const char * const version_msg )
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
+  if( key == 1 ) key = alternative_key( key );			// ^A
   key = ISO_8859::decontrolize( key );
   last_key.shift( key );
   switch( key )
@@ -288,6 +318,7 @@ const char * control_s_loop( int key, const char * const version_msg )
     case '2':
     case '4':
     case '8': msg = w.show_multichar_value( key - '0', false ); break;
+    case 'B': clock_char = ( clock_char != ':' ) ? ':' : ' '; break;
     case 'C': RC::editor_options().show_code_lpos =
                 ( RC::editor_options().show_code_lpos == 1 ) ? 0 : 1;
               show_status_lines(); break;
@@ -353,8 +384,8 @@ const char * user_control_char( const int abort_key )
   while( true )
     {
     key = show_message( prompt1, false, true );
-    if( key == 1 || key == 8 || key == KEY_F(1) )
-      { wait_kbhit(); Menu::help_menu( 1, 8, KEY_F(1) ); }
+    if( key == 8 || key == KEY_F(1) )
+      { wait_kbhit(); Menu::help_menu( 8, KEY_F(1) ); }
     else if( key == 3 || key == '\n' || key == abort_key )	// ^C
       { wait_kbhit(); return 0; }
     else if( key >= 256 || key == '\\' || std::isalnum( key ) ) break;
@@ -387,9 +418,9 @@ const char * user_control_char( const int abort_key )
   }
 
 
-// 'read_key' implements a buffer to speed up pastes, because 'wgetch'
-// always calls 'wrefresh'. 'read_key' also converts '\r' to '\n'.
-//
+/* 'read_key' implements a buffer to speed up pastes, because 'wgetch'
+   always calls 'wrefresh'. 'read_key' also converts '\r' to '\n'.
+*/
 int read_key( const bool wait, const bool get_key )
   {
   enum { buf_size = 1024 };
@@ -471,7 +502,7 @@ extern "C" void Screen::clock_handler( int sig )
     const int h = t->tm_hour;
     clock_string_[0] = ( h < 10 ) ? ' ' : '0' + ( h / 10 );
     clock_string_[1] = '0' + ( h % 10 );
-    clock_string_[2] = ( clock_string_[2] != ':' ) ? ':' : ' ';
+    clock_string_[2] = ( clock_string_[2] != ':' ) ? ':' : clock_char;
     clock_string_[3] = '0' + ( t->tm_min / 10 );
     clock_string_[4] = '0' + ( t->tm_min % 10 );
     if( sig )
@@ -496,11 +527,11 @@ extern "C" void Screen::clock_handler( int sig )
 int Screen::kbhit() { return read_key( false, false ); }
 
 
-// Wait for a keystroke. Returns -1 if no key is available.
-// The clock only blinks when waiting here for a keystroke.
-// The screen is mainly refreshed here (and in clock_handler).
-// This function is the real "main loop" of the program.
-//
+/* Wait for a keystroke. Returns -1 if no key is available.
+   The clock only blinks when waiting here for a keystroke.
+   The screen is mainly refreshed here (and in clock_handler).
+   This function is the real "main loop" of the program.
+*/
 int Screen::wait_kbhit( const int new_clock_limit, const bool get_key,
                         const bool force_refresh )
   {
@@ -520,10 +551,10 @@ int Screen::wait_kbhit( const int new_clock_limit, const bool get_key,
   }
 
 
-// 'history.back()' contains the default result (may be empty), and
-// stores the result (if any).
-// Returns the size of the result string, 0 if none, -1 if aborted.
-//
+/* 'history.back()' contains the default result (may be empty), and
+   stores the result (if any).
+   Returns the size of the result string, 0 if none, -1 if aborted.
+*/
 int Screen::get_string( const std::string & prompt,
                         std::vector< std::string > & history,
                         const int abort_key, const bool usetab )
@@ -547,6 +578,7 @@ int Screen::get_string( const std::string & prompt,
     wmove( stdscr, line, scol - pos );
     do key = wait_kbhit( line ); while( key < 0 );
     if( key == abort_key ) key = 3;
+    if( key == 1 ) key = alternative_key( key );		// ^A
     switch( key )
       {
       case   3 : key = -1; break;				// ^C
@@ -570,9 +602,8 @@ int Screen::get_string( const std::string & prompt,
                    { key = -2; history.back() = name; }
                  }
                  break;
-      case 1:							// ^A
       case 8:							// ^H
-      case KEY_F(1) : Menu::help_menu( 1, 8, KEY_F(1) ); break;
+      case KEY_F(1) : Menu::help_menu( 8, KEY_F(1) ); break;
       case KEY_F(9) : if( Block::valid() )
                         {
                         std::string tmp;
@@ -622,7 +653,7 @@ int Screen::get_string( const std::string & prompt,
         s.erase( prompt.size(), s.size() ); s += history[idx]; scol = s.size();
         } break;
       default: if( key < 256 && ISO_8859::isprint( key ) )
-                 { s.insert( scol++, 1, key ); modified = true; }
+                 { s.insert( s.begin() + scol, key ); ++scol; modified = true; }
       }
     }
 
@@ -827,7 +858,8 @@ int Screen::user_loop( const char * const version_msg )
   bool force_refresh = false;
   while( true )
     {
-    const int key = wait_kbhit( -1, true, force_refresh );
+    int key = wait_kbhit( -1, true, force_refresh );
+    if( key == 1 ) key = alternative_key( key );		// ^A
     if( key < 0 ) continue;
     last_key.set( key );
     const char * msg = 0;
@@ -850,9 +882,8 @@ int Screen::user_loop( const char * const version_msg )
       case 24: if( !Window_vector::close( false ) ) retval = 0; break;	// ^X
       case 25: Window_vector::delete_line(); break;		// ^Y
       case 27: msg = alt_key_loop( key ); break;		// Alt or Esc
-      case 1:							// ^A
       case 8:							// ^H
-      case KEY_F(1) : Menu::help_menu( 1, 8, KEY_F(1) ); break;
+      case KEY_F(1) : Menu::help_menu( 8, KEY_F(1) ); break;
       case KEY_F(2) : msg = Window_vector::save_file( key ); break;
       case KEY_F(3) : msg = Window_vector::load_file( key ); break;
       case KEY_F(4) : if( !Window_vector::load( -1 ) ) retval = 3; break;
