@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 namespace Screen {
@@ -22,7 +22,7 @@ enum Keys { key_up, key_down, key_left, key_right, key_npage, key_ppage,
 
 bool init();
 
-int height();
+int height();			// if 0, screen has not been initialized
 int width();
 int max_windows();
 
@@ -38,19 +38,19 @@ extern "C" void clock_handler( int sig );	// Updates the clock.
 
 int kbhit();			// Returns -1 if no key is available.
 
-// Wait for a keystroke. Returns -1 if no key is available.
-// The clock only blinks when waiting here for a keystroke.
-// The screen is mainly refreshed here (and in clock_handler).
-// This function is the real "main loop" of the program.
-//
+/* Wait for a keystroke. Returns -1 if no key is available.
+   The clock only blinks when waiting here for a keystroke.
+   The screen is mainly refreshed here (and in clock_handler).
+   This function is the real "main loop" of the program.
+*/
 int wait_kbhit( const int new_clock_limit = -1, const bool get_key = true,
                 const bool force_refresh = false );
 
 
-// 'history.back()' contains the default result (may be empty), and
-// stores the result (if any).
-// Returns the size of the result string, 0 if none, -1 if aborted.
-//
+/* 'history.back()' contains the default result (may be empty), and
+   stores the result (if any).
+   Returns the size of the result string, 0 if none, -1 if aborted.
+*/
 int get_string( const std::string & prompt,
                 std::vector< std::string > & history,
                 const int abort_key = 3, const bool usetab = false );
