@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -283,12 +283,16 @@ void Window::move_page( const bool down, const bool scroll )
 void Window::move_vertical( const int lines )
   {
   Point c = bhp->cursor;
+  const int last_line = bhp->buffer().last_line();
 
   if( lines < 0 )
-    { if( c.line == 0 ) { return; } c.line += lines; c.cut(); }
+    {
+    if( c.line == 0 ) return;
+    if( c.line > last_line ) c.line = last_line;
+    else { c.line += lines; c.cut(); }
+    }
   else if( lines > 0 )
     {
-    const int last_line = bhp->buffer().last_line();
     if( c.line == last_line ) return;
     c.line += lines; if( c.line > last_line ) c.line = last_line;
     }
@@ -318,7 +322,7 @@ void Window::scroll_vertical( const int lines )
     { if( tl.line == 0 ) { return; } tl.line += lines; tl.cut(); }
   else if( lines > 0 )
     {
-    int first_line = bhp->buffer().last_line() - ( height_ - 2 );
+    const int first_line = bhp->buffer().lines() - 2;
     if( first_line < 0 || tl.line == first_line ) return;
     tl.line += lines; if( tl.line > first_line ) tl.line = first_line;
     }
@@ -456,7 +460,7 @@ const char * Window::show_utf8_code()
   if( code >= 0 )
     {
     if( p != bhp->pointer ) update_points( p );
-    snprintf( buf, bufsize, "UTF-8 Code %d(0x%04X)  Length %d",
+    snprintf( buf, bufsize, "UTF-8 Code %d(U%04X)  Length %d",
               code, code, len );
     }
   else if( byte_code >= 0 )
