@@ -1,18 +1,18 @@
-/*  GNU moe - My Own Editor
-    Copyright (C) 2005-2019 Antonio Diaz Diaz.
+/* GNU moe - My Own Editor
+   Copyright (C) 2005-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -42,12 +42,12 @@ std::vector< Window > data;
 
 
 const char * already_being_edited( const std::string & name,
-                                   const Buffer & buffer )
+                                   const Buffer * const bufferp = 0 )
   {
   if( name.size() )			// allow multiple unnamed buffers
     for( int i = 0; i < Bufhandle_vector::handles(); ++i )
       if( name == Bufhandle_vector::handle( i ).buffer().name() &&
-          &buffer != &Bufhandle_vector::handle( i ).buffer() )
+          bufferp != &Bufhandle_vector::handle( i ).buffer() )
         {
         int bufsize = 0;
         char * const buf = Screen::line_buf( &bufsize );
@@ -215,9 +215,9 @@ int Window_vector::init()
   }
 
 
-// 'new_size' == 0: reload same number of windows
-// 'new_size' < 0: toggle 1 window <--> max_windows
-//
+/* 'new_size' == 0: reload same number of windows
+   'new_size' < 0: toggle 1 window <--> max_windows
+*/
 int Window_vector::load( int new_size )
   {
   const int handles = Bufhandle_vector::handles();
@@ -270,7 +270,7 @@ const char * Window_vector::change_buffer_name()
   if( Menu::get_filename( "New buffer name (^C to abort): ", name ) < 0 ||
       name == buffer.name() )
     return 0;
-  const char * const msg = already_being_edited( name, buffer );
+  const char * const msg = already_being_edited( name, &buffer );
   if( !msg )
     {
     buffer.name( name );
@@ -286,10 +286,12 @@ const char * Window_vector::load_file( const int abort_key )
   if( Menu::get_filename( "Name of file to edit (^C to abort): ",
                           name, abort_key ) < 0 )
     return 0;
+  Screen::show_feedback( "loading..." );
   const int old_handles = Bufhandle_vector::handles();
   int handle;
   try { handle = Bufhandle_vector::find_or_add_handle( &name, curwin_ + 1 ); }
-  catch( Buffer::Error & e ) { return e.msg; }
+  catch( Buffer::Error & e )
+    { data.back().repaint(); return e.msg; }		// remove feedback
   curwin_ = handle;
   if( windows() > 1 && Bufhandle_vector::handles() > old_handles )
     load( windows() + 1 );
@@ -309,18 +311,19 @@ const char * Window_vector::save_file( const int abort_key )
   {
   const Buffer & buffer = data[curwin_].buffer();
   std::string name( buffer.name() );
+  if( name.empty() && read_only( buffer ) ) return 0;
 
   if( Menu::get_filename( "Name of file to save (^C to abort): ",
       name, abort_key ) <= 0 )
     return 0;
-  const char * const msg = already_being_edited( name, buffer );
+  const char * const msg = already_being_edited( name, &buffer );
   if( msg ) return msg;
   bool done;
   try { done = buffer.save( &name ); }
   catch( Buffer::Error & e ) { return e.msg; }
   if( done )
     {
-    Screen::show_status_lines( &buffer );
+    Screen::show_status_lines( &buffer );	// update modified status
     int bufsize = 0;
     char * const buf = Screen::line_buf( &bufsize );
     snprintf( buf, bufsize, "File '%s' saved",
@@ -361,6 +364,7 @@ int Window_vector::close( bool abort, const bool quiet )
       try { if( !bh.buffer().save( &name ) ) return -1; }
       catch( Buffer::Error & e )
         { if( !quiet ) Screen::show_message( e.msg ); return -1; }
+      if( Bufhandle_vector::handles() <= 1 ) data[curwin_].show_status_line();
       }
     else if( exit_ask && last_handle && !modified )
       {
@@ -570,27 +574,34 @@ const char * Window_vector::read_block()
 const char * Window_vector::write_block()
   {
   if( no_block() ) return 0;
+  const Buffer & buffer = *Block::bufferp();
   std::string name;
   if( Menu::get_filename( "Name of file to write (^C to abort): ", name ) <= 0 )
     return 0;
+  const char * const msg = already_being_edited( name );
+  if( msg ) return msg;
   try {
+    bool done;
     if( !RC::editor_options().rectangle_mode )
-      Block::bufferp()->save( &name, false, false, Block::begin(), Block::end() );
+      done = buffer.save( &name, true, false, Block::begin(), Block::end() );
     else
       {
       Buffer tmp;
       Block::save_block_position();
       Block::copy_block( tmp, tmp.bof() );
       Block::restore_block_position();
-      tmp.save( &name );
+      done = tmp.save( &name );
       }
+    if( !done ) return "Block not written";
     if( RC::editor_options().auto_unmark )
-      {
-      const Buffer * const bufferp = Block::bufferp();
-      Block::reset(); Screen::repaint( bufferp );
-      }
+      { Block::reset(); Screen::repaint( &buffer ); }
     }
   catch( Buffer::Error & e ) { return e.msg; }
+  int bufsize = 0;
+  char * const buf = Screen::line_buf( &bufsize );
+  snprintf( buf, bufsize, "Block written to file '%s'",
+            Screen::cut_filename( name, 24 ) );
+  return buf;
   return 0;
   }
 
@@ -759,24 +770,41 @@ const char * Window_vector::reformat()
     if( bop < bot && buffer.to_cursor( bot ).col > lmargin )
       tmp.Basic_buffer::pputb( dummy, buffer, bop, bot );
     tmp.options.set_lmargin( lmargin );
-    bool space = false;
+    bool period2 = false;	// prev char was period in 2 spaces mode
+    bool pending_space = false;
+    bool change = false;
     for( Point p = bot; p < eop; )
       {
       unsigned char ch = buffer.pgetc( p );
-      if( !std::isspace( ch ) )			// exclude no-break space
-        { tmp.pputc( dummy, ch ); dummy = tmp.eof(); space = false; }
-      else if( !space ) { tmp.pputc( dummy, ' ' ); space = true; }
+      if( std::isspace( ch ) )			// exclude no-break space
+        {
+        if( ( ch != ' ' && ch != '\n' ) ) change = true;
+        pending_space = true;
+        }
+      else
+        {
+        if( pending_space )
+          { pending_space = false; tmp.pputc( dummy, ' ' );
+            if( period2 ) tmp.pputc( dummy, ' ' ); }
+        tmp.pputc( dummy, ch ); dummy = tmp.eof();
+        period2 = ( ch == '.' && buffer.options.two_spaces );
+        }
       if( Screen::kbhit() >= 0 && Screen::wait_kbhit() == 3 )
         return "Aborted";
       }
-    if( space ) tmp.pdelc( dummy, true );
     if( dummy != tmp.bol( dummy ) ) tmp.pputc( dummy, '\n' );
-    bool change = ( tmp.last_line() != ( eop.line - bop.line ) );
+    if( !change ) change = ( tmp.last_line() != ( eop.line - bop.line ) );
     if( !change )
       for( int line = 0; line < tmp.lines(); ++line )
         if( tmp.characters( line ) != buffer.characters( bop.line + line ) &&
             ( !tmp.blank( line ) || !buffer.blank( bop.line + line ) ) )
           { change = true; break; }
+    // detect changes that do not change total number of spaces in line
+    if( !change && buffer.options.two_spaces )
+      for( int line = 0; line < tmp.lines(); ++line )
+        if( tmp.get_line( line ) != buffer.get_line( bop.line + line ) &&
+            ( !tmp.blank( line ) || !buffer.blank( bop.line + line ) ) )
+          { change = true; break; }
     if( change )
       {
       buffer.reset_appendable();
@@ -1058,7 +1086,8 @@ const char * Window_vector::copyright_update()
     {
     int year = 1900 + t->tm_year;
     while( year )
-      { year_string.insert( 0U, 1U, '0' + ( year % 10 ) ); year /= 10; }
+      { year_string.insert( year_string.begin(), '0' + ( year % 10 ) );
+        year /= 10; }
     }
   history.push_back( year_string );
   if( Screen::get_string( "Year to add (^C to abort): ", history ) <= 0 )
@@ -1219,12 +1248,9 @@ const char * Window_vector::search( const int abort_key, const bool again,
       if( len > 0 ) replace_string = replace_history.back();
       }
     }
-  bool backward = backward_opt;
-  switch( toggle )			// leave backward as-is by default
-    {
-    case 2: backward_opt = !backward_opt;	// toggle permanently
-    case 1: backward = !backward;		// toggle this call
-    }
+  if( toggle == 2 ) backward_opt = !backward_opt;	// toggle permanently
+  bool backward = backward_opt;		// leave backward as-is by default
+  if( toggle == 1 ) backward = !backward;		// toggle this call
 
   Point p = curwin().pointer();			// search point
   if( block )
@@ -1246,7 +1272,6 @@ const char * Window_vector::search( const int abort_key, const bool again,
         Screen::show_feedback( ask ? "searching..." : "replacing..." ); }
     Window & w = data[curwin_];
     std::vector< std::string > regex_pieces;
-//    if( replace && empty_replace && !backward ) w.buffer().pnext( p );
     Point p1 = p, p2 = p;
     { const int tmp = Regex::find( w.buffer(), p1, p2, search_string,
                                    regex_pieces, icase, backward );
